/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */
import rst.util.Cronos;
import rst.util.Distribution;
import dd.util.ContingencyTable;
import dd.util.DDTable;

public class DDSample {

	/** This test program has been checked against the DCUBE query
		
		select count(*), max(a.ct.sdiff())
		from PDRULE a
		where a.ct.n1()>0 and a.ct.n2()>0

		with result 35360, 0.8275862068965517
	 */
	public static void main(String[] args) throws Exception 
	{
		final int MINSUPP = 20;
		Cronos c = new Cronos();
		c.start();
		DDTable tb = new DDTable();
		tb.setWorkingDir("ddsample/credit");
		tb.loadFromArff("credit");
		tb.setDiscMetaData("personal_status=female_div_or_dep_or_mar,age=gt_52d6,foreign_worker=yes", "class=bad", MINSUPP);
//		tb.setDiscMetaData("personal_status=female_div_or_dep_or_mar", "class=bad", MINSUPP);
		tb.closedItemset = false;
		tb.extractItemsets(); // only needed if not already extracted
		tb.initScan();
		int nt = 0;
		double largest = -1;
		Distribution dist = new Distribution(-1, 1, 0.1);
		ContingencyTable ct = null, large = null;
		while( (ct = tb.nextCT()) != null) {
			// check if corresponding rule is frequent
			if(ct.a<MINSUPP)
				continue;
			// check if n1 and n2 defined
			if(ct.n1()==0 || ct.n2()==0)
				continue;
			nt++;
			double diff = ct.rd();
			if( diff > largest ) {
				largest = diff;
				large = ct;
			}
			dist.add(diff);
		}
		tb.endScan();
		c.stop();
		System.out.println("Elapsed time (s): " + c.elapsed());
		System.out.println("no. contingency tables: " + nt);
		System.out.println("largest RD: " + largest);
		System.out.println("largest RD\t table: " + large + 
							"\n\tcontext: " + tb.dec.decode(large.frset.toString()) + 
							"\n\tPD: " + tb.dec.decode(large.specs.pdItemset.toString()));
		//dist.output("");
	}
}
