/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package dd.papers.ecmlpkdd2014.frechet;

import java.util.Comparator;

import dd.util.ContingencyTable;

/** Data on unknown contingency table. */
public class FrechetInfo {
	/* Unknown table. */
	public ContingencyTable unknownTable;
	/* Measure lower bound. */
	public double lower;
	/* Measure upper bound. */
	public double upper;
	/* Measure actual value. */
	public double actual;
	
	/* Constructors. */
	public FrechetInfo(ContingencyTable ct, double lower, double upper, double actual) {
		this.unknownTable = ct;
		this.lower = lower;
		this.upper = upper;
		this.actual = actual;
	}

	/** Comparator on the measure lower bound. */
	public static class CompLower implements Comparator<FrechetInfo> {
	    public int compare(FrechetInfo fi1, FrechetInfo fi2) {
	    	// order lexicographic on (lower, support, itemset)
	    	double l1 = fi1.lower;
	    	double l2 = fi2.lower;
	    	if(l1<l2)
	    		return -1;
	    	if(l1>l2)
	    		return 1;
	    	int supp1 = fi1.unknownTable.n();
	    	int supp2 = fi2.unknownTable.n();
	    	if(supp1<supp2)
	    		return -1;
	    	if(supp1>supp2)
	    		return 1;
	    	return fi1.unknownTable.frset.compareTo(fi2.unknownTable.frset);
	    }
	}
}
