/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package dd.papers.ecmlpkdd2014.padd;
import java.io.PrintWriter;
import java.util.Random;
import java.util.TreeSet;

import rst.table.DoubleColumn;
import rst.table.Table;
import rst.util.data.Bitmap;
import rst.util.Cronos;
import rst.util.Distribution;
import rst.util.IOUtil;
import dd.papers.ecmlpkdd2014.frechet.FrechetInfo;
import dd.util.ContingencyTable;
import dd.util.DDTable;

/** PADD Attacks. */
public class PADDAttack {

	/** Test. */
	public static void main(String[] args) throws Exception 
	{
		attack("papers/ecmlpkdd2014/padd", "creditsampleL4", "personal_status=female_div_or_dep_or_mar", "class=bad", 20, 10000);
		attack("papers/ecmlpkdd2014/padd", "adultsample", "race=NonWhite", "income=-50K", 48, 10000);
	}

	/** Attack a dataset. */
	public static void attack(String wdir, String dataset, String PD, String dec,
							int MINSUPP, int TOPTABLES) throws Exception 
	{
		Cronos c = new Cronos();
		c.start();
		DDTable tb = new DDTable();
		tb.setWorkingDir(wdir+"/"+dataset);
		tb.loadFromArff(dataset, true);
		tb.setDiscMetaData(PD, dec, MINSUPP);
		tb.closedItemset = true;
		tb.extractItemsets("GID"); // exclude GID attribute in the extraction
		tb.initScan();
		Table tb2 = Table.loadFromArff(wdir+"/"+dataset+"/"+dataset+".arff");
		DoubleColumn gidcol = tb2.getAttDouble("GID");
		TreeSet<FrechetInfo> top = new TreeSet<FrechetInfo>( new FrechetInfo.CompLower() ); // best inferred table
		int nt = 0;
		ContingencyTable unknown;
		Distribution distRDdiff = new Distribution(0, 2, 0.01);
		Distribution distRD = new Distribution(-1, 1, 0.01);
		Bitmap negativeCover = tb.getCoverDecisionItem();
		Random ran = new Random(System.currentTimeMillis());
		while( (unknown = tb.nextCT()) != null) {
			if( unknown.n1() == 0 || unknown.n2()==0 )
				continue; // if p1 or p2 are undefined
//			System.out.println(unknown);
			nt++;
			int n = unknown.n();
			int n1 = unknown.n1(); // simulate background knowledge
			int n2 = n - n1;
			int m1 = unknown.m1();
			Bitmap bcover = tb.getCover(unknown.frset);
			TreeSet<Integer> gidswithneg = new TreeSet<Integer>();
			TreeSet<Integer> gidswithpos = new TreeSet<Integer>();
			for(int i=0; i<n; ++i) {
				int pos = bcover.getByRank(i);
				int gid = gidcol.get(pos).intValue();
				boolean negdec = negativeCover.get(pos);
				if(negdec)
					gidswithneg.add(gid);
				else
					gidswithpos.add(gid);
			}
			int sumneg = gidswithneg.size();
			int sumpos = gidswithpos.size();
//			if(sumneg+sumpos<n)
//				System.out.println("here");
			int fmax = Math.min(n1,m1);
			int fmin = Math.max(n1+m1-n,0);
//			System.out.println("Fa = " + fmax + " >= a >= " + fmin);
//			System.out.println("sumneg = " + sumneg + " sumpos = " + sumpos);
			
			int au = Math.min(fmax,sumneg), al =  Math.max(fmin,n1 - sumpos);
			int al2 = ran.nextInt(unknown.a+1);
			if(al<al2)
				al = al2;
//			System.out.println("RFa = " + au + " >= a >= " + al);

			int bl = n1 - au, bu = n1 - al;
			int cl = m1 - au, cu = m1 - al;
			//System.exit(0);
//			int cl2 = ran.nextInt(unknown.c+1);
//			if(cl<cl2)
//				cl = cl2;
			int dl = n2 - cu, du = n2 - cl;
			
			ContingencyTable lower = new ContingencyTable(al, bl, cl, dl);
			ContingencyTable upper = new ContingencyTable(au, bu, cu, du);
			double RDlower = lower.a/(double)n1 - upper.c/(double)n2;
			double RDupper = upper.a/(double)n1 - lower.c/(double)n2;
			double actual = unknown.rd();
//			System.out.println(lower + "\t" + upper + "\t" + RDlower  + "\t" + RDupper + "\t" + actual);
			distRD.add(actual);
			double RDfrechetRange = (fmax/(double)n1 - (m1-fmax)/(double)n2) -  (fmin/(double)n1 - (m1-fmin)/(double)n2);
			distRDdiff.add(RDfrechetRange - (RDupper-RDlower));
			FrechetInfo fi = new FrechetInfo(unknown, RDlower, RDupper, actual);
			top.add(fi);
			// keep only best tables
			if( nt > TOPTABLES )
				top.remove(top.first()); // best inferred bounds
		}
		tb.endScan();
		System.out.println("no. contingency tables: " + nt);
		final String outfile = wdir+"/plots/padd_" + dataset + "_" + MINSUPP;
		distRDdiff.output(outfile+ ".dist");
		PrintWriter out = IOUtil.getWriter(outfile+ ".data");
		// output best inferred tables and their distribution of RD lower bound
		int rank=0;
		Distribution distTot = new Distribution(-1, 1, 0.01);
		Distribution distZero = new Distribution(-1, 1, 0.01);
		Distribution distZeroFrechet = new Distribution(-1, 1, 0.01);
		for(FrechetInfo fi : top.descendingSet()) {
			++rank;
			out.println("" + rank + "\t" + fi.lower  + "\t" + fi.upper + "\t" + fi.actual
					+ "\t" + fi.unknownTable);
			distTot.add(fi.lower);
			if(fi.upper == fi.lower) {
				distZero.add(fi.lower);

				int n = fi.unknownTable.n();
				int n1 = fi.unknownTable.n1();
				int n2 = n - n1;
				int m1 = fi.unknownTable.m1();
				int afmax = Math.min(n1,m1);
				int afmin = Math.max(n1+m1-n,0);
				int cfmin = m1 - afmax, cfmax = m1 - afmin;
				double RDlower = afmin/(double)n1 - cfmax/(double)n2;
				double RDupper = afmax/(double)n1 - cfmin/(double)n2;
				if(RDlower == RDupper)
					distZeroFrechet.add(fi.lower);
			}
		}
		out.close();
		out = IOUtil.getWriter(outfile+ ".aggr");
		int tot = 0;
		int zero = 0;
		int zeroF = 0;
		int overall = 0;
	    for(int i=distTot.count.length-1; i>=0;--i) {
	    	int n = distTot.count[i];
	    	if( n == 0 ) 
	    		continue;
    		double v = i*distTot.step+distTot.min;
    		tot += n;
	    	zero += distZero.count[i];
	    	zeroF += distZeroFrechet.count[i];
	    	overall += distRD.count[i];
			out.println("" + v + "\t" + tot  + "\t" + zero  + "\t" + overall+ "\t" + zeroF);
	    }
	    out.close();
		c.stop();
		System.out.println("Elapsed time (s): " + c.elapsed());
	}
}
