/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package dd.util;

import java.io.*;
import java.util.*;

/** Cache of T values for contingency tables. */
public class CTCache<T> implements Serializable 
{
	/** Serial ID. */
	private static final long serialVersionUID = 1564568942724654006L;
	
	/** Compact contingency table. */
	private class I4 implements Comparable<I4>, Serializable 
	{
		private static final long serialVersionUID = 2257651695729333075L;
		public int a, b, c, d;
		public I4() {}
		public I4(int a, int b, int c, int d) {
			this.a = a;
			this.b = b;
			this.c = c;
			this.d = d;
		}
		public int compareTo(I4 oc) {
			if( a > oc.a )
				return 1;
			if( a < oc.a )
				return -1;
			if( b > oc.b )
				return 1;
			if( b < oc.b )
				return -1;
			if( c > oc.c )
				return 1;
			if( c < oc.c )
				return -1;
			if( d > oc.d )
				return 1;
			if( d < oc.d )
				return -1;
			return 0;
		}
	}
	
	/** Utility variable. */
	private I4 i4 = new I4();
	/** Cache. */
	private TreeMap<I4, T> cache;
	
	/** Constructor. */
	public CTCache()
	{
		cache = new TreeMap<I4, T>();
	}
	
	/** Get cached value, if any. */
	public T get(int a, int b, int c, int d) 
	{
		i4.a = a; i4.b = b; i4.c = c; i4.d = d;
		return cache.get(i4);
	}
	
	/** Set cached value. */
	public void put(int a, int b, int c, int d, T v) {
		cache.put(new I4(a,b,c,d), v);
	}
}
