/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package dd.util;

import java.util.HashSet;
import rst.pp.fimi.Decode;
import rst.util.data.Bitmap;

/** Specifier of a contingency table. */
public class CTSpecs {
	/** PD Itemset */
	public final FrequentItemset pdItemset;
	/** Cover of PD itemset */
	public final Bitmap coverPD;
	/** Atts in PD Itemset */
	public HashSet<Integer> pdAtts;
	/** Decode items */
	private Decode dec;
	/** PD Itemset percentage in the overall dataset */
	public final double avgProtected;
	/** DecisionItem percentage in the overall dataset */
	public final double avgNegative;

	/** Constructor. */
	public CTSpecs(Decode dec, FrequentItemset pdItemset, Bitmap coverPD, double avgProtected, double avgNegative) {
		this.pdItemset = pdItemset;
		this.coverPD = coverPD;
		this.avgProtected = avgProtected; 
		this.avgNegative = avgNegative;
		this.dec = dec;
		this.pdAtts = new HashSet<Integer>();
		for(int i: pdItemset.getItems())
			pdAtts.add( dec.getAttribute(i) );
	}
	
	/** True if pdItemset includes at least one attribute present in parameter. */
	public boolean includeAttsFrom(FrequentItemset iset) {
		for(int i: iset.getItems())
			if( pdAtts.contains( dec.getAttribute(i) ) )
				return true;
		return false;
	}
}
