/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.util;

import java.util.ArrayList;

/** Generate subsets. */
public class SubsetGen<T> {
	/** Set of items. */
	private ArrayList<T> set;
	/** Subsets of items. */
	private ArrayList<ArrayList<T>> subsets;
	/** Constructor. */
	public SubsetGen(ArrayList<T> set) {
		this.set = set;
		subsets = new ArrayList<ArrayList<T>>();
		subset(new ArrayList<T>(), set.size());
	}
	/** Return subsets. */
	public ArrayList<ArrayList<T>> getSubsets() {
		return subsets;
	}
	/** Generate subsets. */
	private void subset(ArrayList<T> sset, int n) {
		if(n==0) {
			subsets.add(sset);
			return;
		}
		ArrayList<T> sset2 = new ArrayList<T> (sset);
		sset.add(set.get(--n));
		subset(sset2, n);		
		subset(sset, n);
	}
	/** Test. */
	public static void main(String[] arg) {
		ArrayList<String> set = new ArrayList<String>();
		set.add( "a" ); set.add( "b" );
		set.add( "c" ); set.add( "d" );
		SubsetGen<String> g = new SubsetGen<String>(set);
		System.out.println(g.getSubsets());
	}

}
