/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.util.data;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.TreeMap;

import rst.util.IOUtil;

/** Code values of type T into integers. */
public class EntityCode<T> {
	/** Coding. */
	protected TreeMap<T, Integer> code;
	/** Decoding. */
	protected ArrayList<T> decode;
	/** Constructor. */
	public EntityCode() {
		code = new TreeMap<T, Integer>();
		decode = new ArrayList<T>();
	}
	/** Get code (add a new code if there is no one). */
	public int getCode(T k) {
		Integer v = code.get(k);
		if(v==null) {
			v = code.size();
			code.put(k, v);
			decode.add(k);
		}
		return v;
	}
	/** Number of coded values. */
	public int size() {
		return code.size();
	}
	/** Decode. */
	public T decode(int v) {
		return decode.get(v);
	}
	/** Save codes to a CSV file. */
	public void save(String fileName) throws IOException {
		PrintWriter out = IOUtil.getWriter(fileName);
		int n = decode.size();
		for(int i=0; i<n; ++i)
			out.println(i+"\t"+decode.get(i));
		out.close();
	}
}
